@@ -14,28 +14,47 @@ public abstract class JodaDateSerializerBase<T> extends JodaSerializerBase<T>
 {
     private static final long serialVersionUID = 1L;
 
-    protected final JacksonJodaDateFormat _format;
+    // // Since 2.9
 
-    /**
-     * Marker flag that indicates that timestamp-style format uses JSON arrays,
-     * not integral numbers.
-     * 
-     * @since 2.5
-     */
-    protected final boolean _usesArrays;
+    protected final static int FORMAT_STRING = 1;
+    protected final static int FORMAT_TIMESTAMP = 2;
+    protected final static int FORMAT_ARRAY = 3;
+
+    protected final JacksonJodaDateFormat _format;
 
     protected final SerializationFeature _featureForNumeric;
 
+    /**
+     * Shape to use for generic "use numeric" feature (instead of more specific
+     * JsonFormat.shape).
+     *
+     * @since 2.9
+     */
+    protected final int _defaultNumericShape;
+
+    /**
+     * Marker set to non-0 if (and only if) property or type override exists.
+     *
+     * @since 2.9
+     */
+    protected final int _shapeOverride;
+
     protected JodaDateSerializerBase(Class<T> type, JacksonJodaDateFormat format,
-            boolean usesArrays, SerializationFeature numericFeature)
+            SerializationFeature numericFeature,
+            int defaultNumericShape, int shapeOverride)
     {
         super(type);
         _format = format;
-        _usesArrays = usesArrays;
         _featureForNumeric = numericFeature;
+        _defaultNumericShape = defaultNumericShape;
+        _shapeOverride = shapeOverride;
     }
 
-    public abstract JodaDateSerializerBase<T> withFormat(JacksonJodaDateFormat format);
+    /**
+     * @since 2.9
+     */
+    public abstract JodaDateSerializerBase<T> withFormat(JacksonJodaDateFormat format,
+            int shapeOverride);
 
     @Override
     public boolean isEmpty(SerializerProvider prov, T value) {
@@ -48,28 +67,34 @@ public abstract class JodaDateSerializerBase<T> extends JodaSerializerBase<T>
     {
         JsonFormat.Value ann = findFormatOverrides(prov, property, handledType());
         if (ann != null) {
+            int shapeOverride = 0;
             JacksonJodaDateFormat format = _format;
 
             Boolean useTimestamp;
 
             // Simple case first: serialize as numeric timestamp?
-            if (ann.getShape().isNumeric()) {
+            final JsonFormat.Shape shape = ann.getShape();
+            if (shape.isNumeric()) {
                 useTimestamp = Boolean.TRUE;
-            } else if (ann.getShape() == JsonFormat.Shape.STRING) {
+                shapeOverride = FORMAT_TIMESTAMP;
+            } else if (shape == JsonFormat.Shape.STRING) {
                 useTimestamp = Boolean.FALSE;
-            } else if (ann.getShape() == JsonFormat.Shape.ARRAY) {
+                shapeOverride = FORMAT_STRING;
+            } else if (shape == JsonFormat.Shape.ARRAY) {
                 // 17-Nov-2014, tatu: also, arrays typically contain non-string representation
                 useTimestamp = Boolean.TRUE;
+                shapeOverride = FORMAT_ARRAY;
             } else  {
                 useTimestamp = null;
+                shapeOverride = 0;
             }
             // must not call if flag defined, to rely on defaults:
             if (useTimestamp != null) {
                 format = format.withUseTimestamp(useTimestamp);
             }
             format = format.with(ann);
-            if (format != _format) {
-                return withFormat(format);
+            if ((format != _format) || (shapeOverride != _shapeOverride)) {
+                return withFormat(format, shapeOverride);
             }
         }
         return this;
@@ -77,38 +102,46 @@ public abstract class JodaDateSerializerBase<T> extends JodaSerializerBase<T>
 
     @Override
     public JsonNode getSchema(SerializerProvider provider, java.lang.reflect.Type typeHint) {
-        if (_useTimestamp(provider)) {
-            return createSchemaNode(_usesArrays ? "array" : "number", true);
+        switch (_serializationShape(provider)) {
+        case FORMAT_TIMESTAMP:
+            return createSchemaNode("array", true);
+        case FORMAT_ARRAY:
+            return createSchemaNode("number", true);
+        case FORMAT_STRING:
+        default:
+            return createSchemaNode("string", true);
         }
-        return createSchemaNode("string", true);
     }
 
     @Override
-    public void acceptJsonFormatVisitor(JsonFormatVisitorWrapper visitor, JavaType typeHint) throws JsonMappingException
+    public void acceptJsonFormatVisitor(JsonFormatVisitorWrapper visitor, JavaType typeHint)
+            throws JsonMappingException
     {
-        _acceptJsonFormatVisitor(visitor, typeHint, _useTimestamp(visitor.getProvider()));
-    }
-
-    protected void _acceptJsonFormatVisitor(JsonFormatVisitorWrapper visitor, JavaType typeHint,
-            boolean asNumber) throws JsonMappingException
-    {
-        if (asNumber) {
-            if (_usesArrays) {
-                JsonArrayFormatVisitor v2 = visitor.expectArrayFormat(typeHint);
-                if (v2 != null) {
-                    v2.itemsFormat(JsonFormatTypes.INTEGER);
-                }
-            } else {
+        switch (_serializationShape(visitor.getProvider())) {
+        case FORMAT_TIMESTAMP:
+            {
                 JsonIntegerFormatVisitor v2 = visitor.expectIntegerFormat(typeHint);
                 if (v2 != null) {
                     v2.numberType(JsonParser.NumberType.LONG);
                     v2.format(JsonValueFormat.UTC_MILLISEC);
                 }
             }
-        } else {
-            JsonStringFormatVisitor v2 = visitor.expectStringFormat(typeHint);
-            if (v2 != null) {
-                v2.format(JsonValueFormat.DATE_TIME);
+            break;
+        case FORMAT_ARRAY:
+            {
+                JsonArrayFormatVisitor v2 = visitor.expectArrayFormat(typeHint);
+                if (v2 != null) {
+                    v2.itemsFormat(JsonFormatTypes.INTEGER);
+                }
+            }
+            break;
+        case FORMAT_STRING:
+        default:
+            {
+                JsonStringFormatVisitor v2 = visitor.expectStringFormat(typeHint);
+                if (v2 != null) {
+                    v2.format(JsonValueFormat.DATE_TIME);
+                }
             }
         }
     }
@@ -119,14 +152,25 @@ public abstract class JodaDateSerializerBase<T> extends JodaSerializerBase<T>
     /**********************************************************
      */
 
-    protected boolean _useTimestamp(SerializerProvider provider) {
-        return _format.useTimestamp(provider, _featureForNumeric);
-    }
-
     /**
      * @since 2.8
      */
     protected boolean writeWithZoneId(SerializerProvider provider) {
         return _format.shouldWriteWithZoneId(provider);
     }
+
+    /**
+     * @since 2.9
+     */
+    protected int _serializationShape(SerializerProvider provider) {
+        int shape = _shapeOverride;
+        if (shape == 0) {
+            if (_format.useTimestamp(provider, _featureForNumeric)) {
+                shape = _defaultNumericShape;
+            } else {
+                shape = FORMAT_STRING;
+            }
+        }
+        return shape;
+    }
 }
